  /*
Copyright 2003 by the University of Edinburgh
Code by Dan Mossop.

Display.java

Creates the simulation display window. It also handles all the Java3d code used to display the simulation objects.

*/
package code.model.display;

import code.model.structure.*;
import code.model.display.canvas.*;
import code.model.storage.*;

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;

import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class Display
{
        TransformGroup simulationGroup;
        Vector particleRefs;

        float scale;

        JFrame mainWindow;

        public Display(Canvas3D canvas3d, Point loc, Dimension dim)
        {

                //--- Initialise the display window ---//

                // Get the current look and feel
                try
                {
                        UIManager.setLookAndFeel(
                                UIManager.getSystemLookAndFeelClassName());
                } catch (Exception e) {}

                // Create a JFrame
                mainWindow = new JFrame("Protein Simulation Package - Dan Mossop, 2002.");
                mainWindow.setSize(dim.width, dim.height);
                mainWindow.setLocation(loc);
                mainWindow.getContentPane().setLayout(new BorderLayout());
                mainWindow.addWindowListener(new WindowAdapter()
                {
                        public void windowClosing(WindowEvent e) {System.exit(0);}
                });


		// Add simulation screen
		SimulationCanvas simulationCanvas = new SimulationCanvas(canvas3d);
		mainWindow.getContentPane().add(simulationCanvas, BorderLayout.CENTER);
                simulationGroup = simulationCanvas.getSimulationGroup();

                mainWindow.setVisible(true);

                particleRefs = new Vector();
                scale = 1f;
        }

        public void dispose()
        {
                mainWindow.dispose();
        }

        public Point getLocation()
        {
                return mainWindow.getLocation();
        }

        public Dimension getDimensions()
        {
                return new Dimension(mainWindow.getWidth(), mainWindow.getHeight());
        }

        public void initialiseParticles(String[] particleIds)
        {
                for (int i=simulationGroup.numChildren()-1; i>2; i--)
                {
                        simulationGroup.removeChild(i);
                }

                for (int i=0; i<particleIds.length; i++)
                {
                        if (particleIds[i] != "")
                        {
                                createJ3dObject(particleIds[i]);
                        }
                }
        }

        // Create a default particle and store the hashcode referring to it
	public void createJ3dObject(String id)
	{
	        Appearance a = new Appearance();

                        // Enable Z-buffering to ensure correct object order
                        RenderingAttributes ra = new RenderingAttributes();
                        ra.setCapability(RenderingAttributes.ALLOW_VISIBLE_READ);
                        ra.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);
                        ra.setVisible(false);

                        ra.setDepthBufferEnable(true);
                        a.setRenderingAttributes(ra);
                        a.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_READ);
                        a.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_WRITE);

                        // Set the transparency of the object
                        TransparencyAttributes ta = new TransparencyAttributes();
                        ta.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
                        ta.setTransparencyMode(ta.FASTEST);
                        ta.setTransparency(0.0f);
                        a.setTransparencyAttributes(ta);
                        a.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
                        a.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);

                        a.setCapability(Appearance.ALLOW_MATERIAL_READ);
                        a.setCapability(Appearance.ALLOW_MATERIAL_WRITE);

                        a.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_READ);
                        a.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_WRITE);

                        Material c = new Material();
                        c.setLightingEnable(true);

                        a.setMaterial(c);

                        Primitive primitive = new Sphere(0, a);
                        primitive.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
                        primitive.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
                        Shape3D shape = primitive.getShape(0);
                        shape.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
                        shape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);


                        Transform3D trans = new Transform3D();
                        TransformGroup transGroup = new TransformGroup(trans);
                        transGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                        transGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
                        transGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
                        transGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
                        transGroup.addChild(primitive);

                        BranchGroup bg = new BranchGroup();
                        bg.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
                        bg.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
                        bg.setCapability(BranchGroup.ALLOW_DETACH);
                        bg.addChild(transGroup);

                        bg.compile();
                        simulationGroup.addChild(bg);
                        particleRefs.addElement(id);
	}

        public void showParticles(String[] particleIds)
        {
                for (int partNo = 0; partNo<particleIds.length; partNo++)
                {
                        for (int i=0; i<simulationGroup.numChildren()-3; i++)
                        {
                                if (particleIds[partNo].equals((String) (particleRefs.elementAt(i))))
                                {
                                        BranchGroup bg = (BranchGroup) (simulationGroup.getChild(i+3));
                                        TransformGroup tg = (TransformGroup) (bg.getChild(0));
                                        Transform3D trans = new Transform3D();
                                        Primitive primitive = (Primitive) (tg.getChild(0));
                                        Appearance app = primitive.getAppearance();
                                        app.getRenderingAttributes().setVisible(true);

                                }
                        }
                }
        }

        Primitive getJ3dShape(String shape, float[] dimensions, Appearance a)
        {
                if (shape.equals("sphere"))
                {
                        return new Sphere(dimensions[0]*scale, a);
                }
                else if (shape.equals("cylinder"))
                {
                        return new Cylinder(dimensions[0]*scale, dimensions[1]*scale, a);
                }
                else if (shape.equals("box"))
                {
                        return new com.sun.j3d.utils.geometry.Box(dimensions[0]*scale, dimensions[1]*scale, dimensions[2]*scale, a);
                }
                return null;
        }

        public void setColour(String p, Color3f col)
        {
                for (int i=0; i<simulationGroup.numChildren()-3; i++)
                {
                        if (p.equals((String) (particleRefs.elementAt(i))))
                        {
                                BranchGroup bg = (BranchGroup) (simulationGroup.getChild(i+3));
                                TransformGroup tg = (TransformGroup) (bg.getChild(0));
                                Transform3D trans = new Transform3D();
                                Primitive primitive = (Primitive) (tg.getChild(0));
                                Appearance app = primitive.getAppearance();

                                Material c = new Material();
		                		c.setLightingEnable(true);
                                c.setAmbientColor(col);
                                c.setEmissiveColor(new Color3f(0f,0f,0f));
                                c.setDiffuseColor(new Color3f(col.x*0.15f, col.y*0.15f, col.z*0.15f));
                                c.setSpecularColor(col);
                                c.setShininess(128.0f);
                                app.setMaterial(c);
                        }
                }

        }

        public void setAlpha(String p, float alpha)
        {
                for (int i=0; i<simulationGroup.numChildren()-3; i++)
                {
                        if (p.equals((String) (particleRefs.elementAt(i))))
                        {
                                BranchGroup bg = (BranchGroup) (simulationGroup.getChild(i+3));
                                TransformGroup tg = (TransformGroup) (bg.getChild(0));
                                Transform3D trans = new Transform3D();
                                Primitive primitive = (Primitive) (tg.getChild(0));
                                Appearance app = primitive.getAppearance();
                                TransparencyAttributes ta =
                                        new TransparencyAttributes(TransparencyAttributes.SCREEN_DOOR, alpha);
                                app.setTransparencyAttributes(ta);
                        }
                }

        }

        public void setPrimitive(String p, String shape, float[] dimensions)
        {
                for (int i=0; i<simulationGroup.numChildren()-3; i++)
                {
                        if (p.equals((String) (particleRefs.elementAt(i))))
                        {
                                BranchGroup bgOld = (BranchGroup) (simulationGroup.getChild(i+3));
                                TransformGroup tgOld = (TransformGroup) (bgOld.getChild(0));
                                Transform3D trans = new Transform3D();
                                tgOld.getTransform(trans);
                                Primitive primOld = (Primitive) (tgOld.getChild(0));
                                Appearance app = primOld.getAppearance();
                                simulationGroup.removeChild(i+3);

                                Primitive primNew = getJ3dShape(shape, dimensions, app);
                                primNew.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
                                primNew.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
                                Shape3D shape3d = primNew.getShape(0);
                                shape3d.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
                                shape3d.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);


                                TransformGroup tgNew = new TransformGroup(trans);
		                		tgNew.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                                tgNew.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
                                tgNew.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
                                tgNew.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
                                tgNew.addChild(primNew);

                                BranchGroup bgNew = new BranchGroup();
                                bgNew.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
                                bgNew.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
                                bgNew.setCapability(BranchGroup.ALLOW_DETACH);
                                bgNew.addChild(tgNew);

                                simulationGroup.insertChild(bgNew, i+3);

                                return;
                        }
                }
                return;
        }

        public void setOrientation(String p, AxisAngle4f orient)
        {				 
                for (int i=0; i<simulationGroup.numChildren()-3; i++)
                {
                        if (p.equals((String) (particleRefs.elementAt(i))))
                        {
                                BranchGroup bg = (BranchGroup) (simulationGroup.getChild(i+3));
                                TransformGroup tg = (TransformGroup) (bg.getChild(0));
                                Transform3D trans = new Transform3D();
                                tg.getTransform(trans);
                         		//Vector3f translation = new Vector3f();
                         		//trans.get(translation);
                         
                         		//Transform3D newTrans = new Transform3D();
                                trans.setRotation(orient);
                                //trans.setTranslation(translation);
                                
                                tg.setTransform(trans);
                                return; 
                        }
                }
                return;
        }

        public void translateParticle(String id, Vector3f trans)
        {
                int index = getParticleIndex(id);
                BranchGroup bg = (BranchGroup) (simulationGroup.getChild(index+3));
                TransformGroup tg = (TransformGroup) (bg.getChild(0));
                Transform3D transform = new Transform3D();
                tg.getTransform(transform);
                Vector3f com = new Vector3f();
                trans.get(com);

                // scale down
                com.x /= scale;
                com.y /= scale;
                com.z /= scale;

                com.x += trans.x;
                com.y += trans.y;
                com.z += trans.z;

                setCentreOfMass(id, com);
        }

        public void setCentreOfMass(String p, Vector3f com)
        {
                for (int i=0; i<simulationGroup.numChildren()-3; i++)
                {
                        if (p.equals((String) (particleRefs.elementAt(i))))
                        {
                                BranchGroup bg = (BranchGroup) (simulationGroup.getChild(i+3));
                                TransformGroup tg = (TransformGroup) (bg.getChild(0));
                                Transform3D trans = new Transform3D();
                                tg.getTransform(trans);

                                Vector3f scaled = new Vector3f();
                                // scale up
                                scaled.x = com.x * scale;
                                scaled.y = com.y * scale;
                                scaled.z = com.z * scale;

                                trans.setTranslation(scaled);
                                tg.setTransform(trans);
                                return;
                        }
                }
                return;
        }

        public int getParticleIndex(String id)
        {
                for (int i=0; i<simulationGroup.numChildren()-3; i++)
                {
                        if (id.equals((String) (particleRefs.elementAt(i))))
                        {
                                return i;
                        }
                }
                return -1;
        }

        public void setScale(float s)
        {
                scale = s;
        }

        public float getScale()
        {
                return scale;
        }
}
