 /*
Copyright 2003 by the University of Edinburgh
Code by Dan Mossop.

FileHandler.java

Responsible for loading the simulation files and bridging between the XML files and the main simulation object format.

*/

package code.model.reader;

import neuroml.core.*;
import neuroml.util.*;

import java.util.*;

import gui.*;

import code.model.structure.*;
import code.model.reader.xmlobjects.XMLParticleTemplate;
import code.model.reader.xmlobjects.*;
import code.model.reader.XMLList;

public class FileHandler
{
   public FileHandler()
   {
   }

   /////////// Output //////////////

   public static void toFile(String filename, Vector modelVect)
   {
        Vector xmlVect = convertToXMLObjects(modelVect);

        XMLOut.toFile(new XMLList(xmlVect), filename);
   }

   private static Vector convertToXMLObjects(Vector modelVect)
   {
        Vector xmlVect = new Vector();

        for (int i=0; i<modelVect.size(); i++)
        {
                Object obj = modelVect.elementAt(i);

                if (obj instanceof ParticleTemplate)
                {
                        xmlVect.addElement(new XMLParticleTemplate((ParticleTemplate) obj));
                }
                else if (obj instanceof SimObjectTemplate)
                {
                        xmlVect.addElement(new XMLSimObjectTemplate((SimObjectTemplate) obj));
                }
                else if (obj instanceof SimObject)
                {
                        xmlVect.addElement(new XMLSimObject((SimObject) obj));
                }
                else if (obj instanceof BondTemplate)
                {
                                xmlVect.addElement(new XMLBondTemplate((BondTemplate) obj));
                }
                else if (obj instanceof RuleTemplate)
                {
                        xmlVect.addElement(new XMLRuleTemplate((RuleTemplate) obj));
                }
                else if (obj instanceof Landscape)
                {
                        xmlVect.addElement(new XMLLandscape((Landscape) obj));
                }
        }

        return xmlVect;
   }
   ////////// Input ///////////

   // load in the specified file
   public static Vector fromFile(String filename)
   {
        XMLIn.addKnownPackage("code");
        XMLIn.addKnownPackage("code.model");
        XMLIn.addKnownPackage("code.model.reader");
        XMLIn.addKnownPackage("code.model.reader.xmlobjects");
        XMLList xmlList = (XMLList) XMLIn.loadXML(filename);

        Vector xmlVect = xmlList.toVector();

        return convertFromXMLObjects(xmlVect);
   }

   private static Vector convertFromXMLObjects(Vector xmlVect)
   {
        Vector modelVect = new Vector();

        for (int i=0; i<xmlVect.size(); i++)
        {
                Object xmlObj = xmlVect.elementAt(i);
                if (xmlObj instanceof XMLParticleTemplate)
                {
                        ParticleTemplate pt =(ParticleTemplate) (((XMLParticleTemplate)xmlObj).getObject());
                        modelVect.addElement(pt);
                }
                else if (xmlObj instanceof XMLSimObjectTemplate)
                {
                        SimObjectTemplate sot =(SimObjectTemplate) (((XMLSimObjectTemplate)xmlObj).getObject());
                        modelVect.addElement(sot);
                }
                else if (xmlObj instanceof XMLSimObject)
                {
                        SimObject so =(SimObject) (((XMLSimObject)xmlObj).getObject());
                        modelVect.addElement(so);
                }
                else if (xmlObj instanceof XMLBondTemplate)
                {
                                BondTemplate bt = (BondTemplate) (((XMLBondTemplate) xmlObj).getObject());
                                modelVect.addElement(bt);
                }
                else if (xmlObj instanceof XMLRuleTemplate)
                {
                        RuleTemplate rt = (RuleTemplate) (((XMLRuleTemplate) xmlObj).getObject());
                        modelVect.addElement(rt);
                }
                else if (xmlObj instanceof XMLLandscape)
                {
                        Landscape land = (Landscape) (((XMLLandscape) xmlObj).getObject());
                        modelVect.addElement(land);
                }
                else if (xmlObj instanceof XMLParams)
                {
                        sim.setSimulationSize( ((XMLParams) xmlObj).getSimulationSize());
                }
        }

        return modelVect;
   }
}
