package code.model.reader.xmlobjects;

import code.model.structure.*;
import code.model.utils.*;

import java.util.*;

import javax.vecmath.*;

public class XMLParticleTemplate
{
        public String id;
        public float mass;

        public String shape;
        public neuroml.core.List dimensions = new neuroml.core.List();

        public float colourR = 255f;
        public float colourG = 255f;
        public float colourB = 255f;
        public float alpha = 0f;
        public float bondedColourR = 255f;
        public float bondedColourG = 255f;
        public float bondedColourB = 255f;
        public float bondedAlpha = 0f;

        public String landscapeId;

        public String bondTemplateIds = "";
        public neuroml.core.List bondPointX = new neuroml.core.List();
        public neuroml.core.List bondPointY = new neuroml.core.List();
        public neuroml.core.List bondPointZ = new neuroml.core.List();

        public boolean collide;

        public XMLParticleTemplate()
        {
        }

        public XMLParticleTemplate(ParticleTemplate pt)
        {
                id = pt.getId();
                mass = pt.getMass();

                shape = pt.getShapeName();
                float[] dims = pt.getShapeDimensions();
                for (int i=0; i<dims.length; i++)
                {
                        dimensions.add(new Float(dims[i]));
                }
				
                Color3f col = pt.getColour();
                colourR = col.x;
                colourG = col.y;
                colourB = col.z;
                alpha = pt.getAlpha();

                landscapeId = pt.getLandscapeId();


                Vector bondIds = pt.getBondTemplateIds();
                for (int i=0; i<bondIds.size(); i++)
                {
                        if (i != 0) { bondTemplateIds += ";"; }
                        bondTemplateIds += (String)(bondIds.elementAt(i));
                }

                collide = pt.getCollide();

                Vector3f[] bondPoints = pt.getBondPoints();
                for (int i=0; i<bondPoints.length; i++)
                {
                        for (int j=0; j<3; j++)
                        {
                                Vector3f point = bondPoints[i];
                                bondPointX.add(new Float(point.x));
                                bondPointY.add(new Float(point.y));
                                bondPointZ.add(new Float(point.z));
                        }
                }
        }

        public ParticleTemplate getObject()
        {
                ParticleTemplate pt = new ParticleTemplate(id);
                pt.setMass(mass);

                Object[] dimObjs = dimensions.toArray();
                float[] dims = new float[dimObjs.length];
                for (int i=0; i<dims.length; i++)
                {
                        dims[i] = ((Float) dimObjs[i]).floatValue();
                }

                pt.setShape(shape, dims);

                pt.setColour(new Color3f(colourR, colourG, colourB));
                pt.setBondedColour(new Color3f(bondedColourR, bondedColourG, bondedColourB));
                pt.setAlpha(alpha);
                pt.setBondedAlpha(bondedAlpha);

                pt.setLandscapeId(landscapeId);

                String[] btIds = Functions.split(';', bondTemplateIds);
                Vector bondIds = new Vector();
                for (int i=0; i<btIds.length; i++)
                {
                        bondIds.addElement(btIds[i]);
                }
                pt.setBondTemplateIds(bondIds);

                pt.setCollide(collide);

                Object[] bondPointXObjs = bondPointX.toArray();
                Object[] bondPointYObjs = bondPointY.toArray();
                Object[] bondPointZObjs = bondPointZ.toArray();
                Vector3f[] bondPoints = new Vector3f[bondPointXObjs.length];

                for (int i=0; i < bondPointXObjs.length; i++)
                {
                        bondPoints[i] = new Vector3f (((Float) bondPointXObjs[i]).floatValue(),
                                                                        ((Float) bondPointYObjs[i]).floatValue(),
                                                                        ((Float) bondPointZObjs[i]).floatValue());
                }
                pt.setBondPoints(bondPoints);

                return pt;
        }
}