// -- class for inter-object bonds

package code.model.structure;

import java.util.*;
import javax.vecmath.*;

import code.model.*;
import code.model.storage.*;

public class Bond
{
        String id;
		String templateId;
        String parentParticleId;
        String connectedBondId;
        Vector blockRules;
        Vector dropRules;
        Vector addRules;
		SimulationModel simulation;
		Random rand;

        public Bond(String i, String pp, SimulationModel sim)
        {	
                id = i;
				templateId = "";
                parentParticleId = pp;
				simulation = sim;
				connectedBondId = "";
				rand = new Random();
        }
		
		public String getId()
		{
			return id;
		}
		
		public void setId(String i)
		{
			id = i;
		}
		
		public String getConnectedBond()
		{
			return connectedBondId;
		}
		
		public void setBlockRuleIds(Vector rid)
		{
			blockRules = rid;
		}
		
		public Vector getBlockRuleIds()
		{
			return blockRules;
		}
		
		public void setDropRuleIds(Vector rid)
		{
			dropRules = rid;
		}
		
		public Vector getDropRuleIds()
		{
			return dropRules;
		}
		
		public void setAddRuleIds(Vector rid)
		{
			addRules = rid;
		}
		
		public Vector getAddRuleIds()
		{
			return addRules;
		}
		
		public void setTemplateId(String ti)
		{
			templateId = ti;
		}
		
		public String getTemplateId()
		{
			return templateId;
		}

        public void addRule(String action, String template, String[] ifClause, String[] thenClause)
        {
                // -- create new rule
                Rule rule = new Rule("");
                rule.setRule(action, ifClause, thenClause);

                if (action.equals("block"))
                {
                        blockRules.addElement(rule);
                }
                else if (action.equals("drop"))
                {
                        dropRules.addElement(rule);
                }
                else
                {
                        addRules.addElement(rule);
                }
        }

         // -- called after each object move - on completion the method will have changed the status of the bond,
        // -- if necessary. Returns true if an update is made
        public void update(String[] neighbourParticleIds, Vector newBonds)
        {
                if (!connectedBondId.equals(""))
                {
                        // -- check to see if the current bond is to be dropped
                        // -- if so, force a drop on the connected bond
                        if (checkDropBond())
                        {
                                // -- force the other bond to drop this one
                                Bond other = (Bond)(simulation.getInstance(connectedBondId));
                                other.forceDrop();
                                // -- then drop it at this end
                                forceDrop();
                                return;
                        }
                }

                // start from a random particle to ensure that all particles have an equal chance of being chosen (on average)
                int numParts = neighbourParticleIds.length;
                int start;
                if (numParts <= 0)
                { return;  }
                else if (numParts == 1)
                { start = 0;  }
                else
                { start = rand.nextInt(numParts - 1); }

                // -- test this bond against each particle to see if it can form a bond
                String candBondId = "";
                String result = "";
                for (int i=0; i<numParts; i++)
                {
                        // -- call this bond's canBondTo() method for each neighbour
                        // -- if it can bond, add this to the list of candidate particles
                        String neighbourPart = neighbourParticleIds[(i + start) % numParts];
                        Vector bondIds = ((Particle)(simulation.getInstance(neighbourPart))).getBondIds();
                        boolean canBond = false;

                        int numBonds = bondIds.size();
                        int bondStart;

                        if (numBonds <= 0)
                        { return; }
                        else if (numBonds == 1)
                        { bondStart = 0; }
                        else
                        { bondStart = rand.nextInt(numBonds -1); }

                        for (int j=0; j<bondIds.size(); j++)
                        {
                                String neighbourBondId = (String) (bondIds.elementAt( (j+bondStart) % numBonds ));
                                Bond neighbourBond = (Bond)(simulation.getInstance(neighbourBondId));
                                String result1 =  canBondTo(neighbourBond);
                                String result2 =  neighbourBond.canBondTo(this);
                                if (!result1.equals("fail") && !result2.equals("fail"))
                                {
                                        canBond = true;
                                        candBondId = neighbourBondId;
                                        if (result1.equals("all") && result2.equals("all"))
                                        {
                                                result = "all";
                                        }
                                        else
                                        {
                                                result = "path";
                                        }

                                        String[] newBond = {id, candBondId, result};
                                        newBonds.addElement(newBond);
                                        break;
                                }
                        }
                        //if (canBond)
                        //{
                        //        break;
                        //}
                }

                if (candBondId.equals(""))
                {
                        // no candidate found
                        return;
                }


               //String[] newBond = {id, candBondId, result};
               //newBonds.addElement(newBond);
                return;
        }

        public void bondTo(String successId)
        {
                Bond successBond = (Bond) (simulation.getInstance(successId));

                if (!(connectedBondId.equals("")))
                {
                        Bond oldBond = (Bond) (simulation.getInstance(connectedBondId));
                        oldBond.setConnectedBond("");
                }
                if (!(successBond.getConnectedBond().equals("")))
                {
                        Bond oldBond2 = (Bond) (simulation.getInstance(successBond.getConnectedBond()));
                        oldBond2.setConnectedBond("");
                }
                // -- set that bond's bonded particle to the current bond and
                successBond.setConnectedBond(id);
                // -- set this bond's connected bond to the successful bond.
                setConnectedBond(successId);
        }

        public void setConnectedBond(String cb)
        {
                        connectedBondId = cb;
                        if (!connectedBondId.equals(""))
                        {
                                Particle parent = (Particle) (simulation.getInstance(parentParticleId));
                                ParticleTemplate parentTmpl =
                                                (ParticleTemplate) (simulation.getTemplate(parent.getTemplateId()));
                                Color3f tmplCol = parentTmpl.getBondedColour();
                                simulation.setParticleColour(parentParticleId, tmplCol);
                        }
                        else
                        {
                                Particle parent = (Particle) (simulation.getInstance(parentParticleId));
                                ParticleTemplate parentTmpl =
                                                (ParticleTemplate) (simulation.getTemplate(parent.getTemplateId()));
                                Color3f tmplCol = parentTmpl.getBondedColour();
                                simulation.setParticleColour(parentParticleId, tmplCol);
                        }
        }

        public String getParentParticle()
        {
                return parentParticleId;
        }

        public boolean checkDropBond()
        {
                // -- test each drop rule to determine if the bond should be broken
                for (int i=0; i<dropRules.size(); i++)
                {
                        Bond thatBond = (Bond)(simulation.getInstance(connectedBondId));
                        String dropRuleId = (String)(dropRules.elementAt(i));
                        String result =  ((Rule)(simulation.getInstance(dropRuleId))).test(this, thatBond);
                        if ( result.equals("pass:all") )
                        {
                                // -- a rule has been found which states that the particle should be dropped
                                return true;
                        }
                }
                // -- no rule has been found, so keep the particle
                return false;
        }

        public void forceDrop()
        {
                setConnectedBond("");
                Particle parent = (Particle) (simulation.getInstance(parentParticleId));

                // -- check if the particle is still bonded to other particles
                Vector bondIds = parent.getBondIds();
                for (int i=0; i<bondIds.size(); i++)
                {
                        Bond bond = (Bond) (simulation.getInstance((String)(bondIds.elementAt(i))));
                        if (!(bond.getConnectedBond().equals("")))
                        {
                                return;
                        }
                }

                ParticleTemplate parentTmpl =
                                (ParticleTemplate) (simulation.getTemplate(parent.getTemplateId()));
                Color3f tmplCol = parentTmpl.getColour();
                simulation.setParticleColour(parentParticleId, tmplCol);
        }

        public String canBondTo(Bond thatBond)
        {
                boolean blocked = false;
                // -- check the block rules
                for (int i=0; i<blockRules.size(); i++)
                {
                        String blockRuleId = (String)(blockRules.elementAt(i));
                        //if ( ((Rule)(simulation.getInstance(blockRuleId))).test(this, thatBond) )
                        //{
                        //        // -- the bond has been blocked
                        //        return false;
                        //}
                        String result = ((Rule)(simulation.getInstance(blockRuleId))).test(this, thatBond);
                        if (result.equals("pass:all"))
                        {
                                blocked = true;
                        }
                }

                boolean addPath = false;
                boolean addProb = false;
                // -- check the add rules to see if it can be added
                for (int i=0; i<addRules.size(); i++)
                {
                        String addRuleId = (String)(addRules.elementAt(i));
                        //if ( ((Rule)(simulation.getInstance(addRuleId))).test(this, thatBond) )
                        //{
                        //        // -- the bond can be formed
                        //        return true;
                        //}
                        String result = ((Rule)(simulation.getInstance(addRuleId))).test(this, thatBond);
                        if (result.equals("pass:path"))
                        {
                                addPath = true;
                        }
                        else if (result.equals("pass:all"))
                        {
                                addPath = true;
                                addProb = true;
                        }
                }

                if (blocked && !addPath)
                {
                        return ("fail");
                }
                else if (blocked && addPath)
                {
                        return ("path");
                }
                else if (addPath && addProb)
                {
                        return ("all");
                }
                else if (addPath && !addProb)
                {
                        return ("path");
                }
                else if (!addPath)
                {
                        return ("fail");
                }

                return "fail";
        }
}