// -- a single bonding rule

package code.model.structure;

import code.model.storage.*;
import code.model.*;
import java.util.*;
import javax.vecmath.*;

public class Landscape
{
        String id;
		String type;

        public Landscape(String i)
        {
                id = i;
				type = "flat";        
		}
		
		public void setId(String i)
		{
			id = i;
		}
		
		public String getId()
		{
			return id;
		}
		
		public void setType(String t)
		{
			type=t;
		}
		
		public String getType()
		{
			return type;
		}
		
		public void adjustDisplacement(Vector3f disp, Vector3f pos)
		{
			// -- if the landscapes place any restrictions on the object displacement, then
			// -- adjust the displacement accordingly
			
			if (type.equals("membrane"))
			{
				disp.y = 0f;
				return;
			}
			else if (type.equals("hydrophilic"))
			{
				return;
			}
			else if (type.equals("static"))
			{
				// no displacement
				disp.x = 0f;
				disp.y = 0f;
				disp.z = 0f;
				return;
			}
			else if (type.equals("below membrane"))
			{
				if (disp.y + pos.y > -8f*(float) Math.pow(10.0, -9.0))
				{
					disp.y = 0f;
					return;
				}
				else
				{
					return;
				}
			}
			else
			{
				// the landscape makes no adjustments
				return;
			}
		}
		
		public void adjustRotation(AxisAngle4f orient, Vector3f pos)
		{
			// -- if the landscapes place any restrictions on the object orientation, then
			// -- adjust the orientation accordingly
			
			if (type.equals("membrane"))
			{
				
				if (orient.y == 1f || orient.y == -1f)
				{
					orient.x = 0f;
					orient.z = 0f;
					return;
				}
				else
				{
					orient.angle = 0f;
				}
				
				return;
			}
			else if (type.equals("hydrophilic"))
			{
				return;
			}
			else if (type.equals("static"))
			{
				// no rotation
				orient.angle = 0f;
				return;
			}
			else if (type.equals("below membrane"))
			{
				return;
			}
			else
			{
				// the landscape makes no adjustments
				return;
			}
		}
		
		public void adjustDiffusionAxis(Vector3f axis, Vector3f pos)
		{
			// -- Adjust the diffusion axis if necessary
			
			if (type.equals("membrane"))
			{
				
				axis.set(0f,1f,0f);
				return;
			}
			else if (type.equals("hydrophilic"))
			{
				return;
			}
			else if (type.equals("static"))
			{
				return;
			}
			else if (type.equals("below membrane"))
			{
				return;
			}
			else
			{
				return;
			}
		}

		public float valueAt(Vector3f pos)
		{
			if (type.equals("membrane"))
			{
				return valueAtMembrane(pos);
			}
			else if (type.equals("hydrophilic"))
			{
				return valueAtHydrophilic(pos);
			}
			else if (type.equals("static"))
			{
				return valueAtStatic(pos);
			}
			else if (type.equals("below membrane"))
			{
				return valueAtFlat(pos);
			}
			else
			{
				return valueAtFlat(pos);
			}
		}
		
		public float valueAtStatic(Vector3f pos)
		{
			return 1f;
		}
		
		public float valueAtHydrophilic(Vector3f pos)
		{
			return  -2f * ((float) Math.abs(pos.y / (float) Math.pow(10.0, -9.0)));	
		}
		
		public float valueAtMembrane(Vector3f pos)
		{
			float val = 4f * (float) Math.abs(pos.y / (float) Math.pow(10.0, -9.0));
			if (val < 4f)
			{
				val = 4f;
			}
			return val;
		}
		
		public float valueAtFlat(Vector3f pos)
		{
			return 1f;
		}
}