package code.model.structure;

import java.util.*;

import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;

import javax.media.j3d.*;
import javax.vecmath.*;

public class ParticleTemplate
{
        private String id;
        private float mass;
        private Shape shape;
        private Color3f colour;
        private Color3f bondedColour;
        private float alpha;
        private float bondedAlpha;

        private String landscapeId;

        private Vector3f centreOfMass;
        private AxisAngle4f orientation;
		
        private Vector bondTemplateIds;
        private Vector3f[] bondPoints;

        private boolean collide;

        public ParticleTemplate(String i)
        {
                id = i;
                mass = 0f;
                shape = new Shape();
				
                landscapeId = "";

                centreOfMass = new Vector3f();
                orientation = new AxisAngle4f();
				
                bondTemplateIds = new Vector();
                bondPoints = new Vector3f[0];

                colour = new Color3f();
                bondedColour = new Color3f();

                alpha = 0f;
                bondedAlpha = 0f;

                collide = false;
        }

        public Color3f getBondedColour()
        {
                return bondedColour;
        }

        public void setBondedColour(Color3f c)
        {
                bondedColour = c;
        }
		
        public boolean getCollide()
        {
                return collide;
        }

        public void setCollide( boolean c)
        {
                collide = c;
        }

        public String getLandscapeId()
        {
                return landscapeId;
        }

        public void setLandscapeId(String land)
        {
                landscapeId = land;
        }

        public String getId()
        {
                return id;
        }

        public void setId(String i)
        {
                id = i;
        }
		
        public void setBondTemplateIds(Vector bti)
        {
                bondTemplateIds = bti;
        }

        public Vector getBondTemplateIds()
        {
                return bondTemplateIds;
        }

        public void setBondPoints(Vector3f[] bp)
        {
                bondPoints = bp;
        }

        public Vector3f[] getBondPoints()
        {
                return bondPoints;
        }

        public String toString()
        {
                String bonds = "";
                for (int i=0; i<bondTemplateIds.size(); i++)
                {
                        if (i != 0) {bonds += ", ";}
                        bonds += ((String) (bondTemplateIds.elementAt(i)));
                }
                return id + " (ParticleTemplate)   Bonds: "+bonds;
        }

        public void setMass(float m)
        {
                mass = m;
        }

        public float getMass()
        {
                return mass;
        }

        public void setShape(String n, float[] dim)
        {
                shape.setName(n);
                shape.setDimensions(dim);
        }

        public void setColour(Color3f col)
        {
                colour = col;
        }

        public void setAlpha(float a)
        {
                alpha = a;
        }

        public void setBondedAlpha(float a)
        {
                bondedAlpha = a;
        }

        public float getBondedAlpha()
        {
                return bondedAlpha;
        }

        public void setCentreOfMass(Vector3f com)
        {
                centreOfMass = com;
        }

        public void setOrientation(AxisAngle4f orient)
        {
                orientation = orient;
        }

        public String getShapeName()
        {
                return shape.getName();
        }

        public float[] getShapeDimensions()
        {
                return shape.getDimensions();
        }

        public float getShapeRadius()
        {
                return shape.getRadius();
        }

        public float getShapeVol()
        {
                return shape.getVol();
        }

        public float getAlpha()
        {
                return alpha;
        }
		
        public Color3f getColour()
        {
                return colour;
        }

        public Vector3f getCentreOfMass()
        {
                return centreOfMass;
        }

        public AxisAngle4f getOrientation()
        {
                return orientation;
        }
}

class Shape
{
        String name;
        float[] dimensions;
        float radius;
        float vol;

        Shape()
        {
                name = "";
                dimensions = new float[0];
                radius = 1f;
                vol = 1f;
        }

        ////// Set details //////
        void setName(String n)
        {
                name = n;
                calcRadiusAndVol();
        }

        void setDimensions(float[] dim)
        {
                dimensions = dim;
                calcRadiusAndVol();
        }

        ////// Get details //////
        String getName()
        {
                return name;
        }

        float[] getDimensions()
        {
                return dimensions;
        }
		
        void calcRadiusAndVol()
        {
                if (name.equals("") || dimensions.length == 0)
                {
                        radius = 1f;
                        vol = 1f;
                        return;
                }

                // Set the radius to be that of a sphere with the same volume as this shape
                if (name.equals("sphere"))
                {
                        radius = dimensions[0];
                        vol = (4f / 3f) * (float) Math.PI * radius * radius * radius;
                        return;
                }
                else if (name.equals("cylinder"))
                {
                        // compute equivalent sphere using Rsphere ^ 3 = (3/4) * Rcylinder ^ 2 * Hcylinder
                        float cylinderVolOverPI = dimensions[0] * dimensions[0] * dimensions[1];
                        vol = cylinderVolOverPI * (float) Math.PI;
                        radius = dimensions[0];
                        return;
                }
                else
                {
                        radius = 1f;
                        vol = 1f;
                        return;
                }
        }

        float getVol()
        {
                return vol;
        }

        float getRadius()
        {
                return radius;
        }

}