// -- a single bonding rule

package code.model.structure;

import gui.*;
import code.model.storage.*;
import code.model.*;
import code.model.utils.*;
import java.util.*;
import javax.vecmath.*;

public class Rule
{
        String id;
        String templateId;
        String action;
        String[] ifClause;
        String[] thenClause;
        SimulationModel simulation;

        private Random rand;

        public Rule(String i)
        {
                id = i;
                templateId="";
                action = "none";
                ifClause = new String[0];
                thenClause = new String[0];
                rand = new Random();
        }

        public void setTemplateId(String tmpId)
        {
                templateId = tmpId;
        }

        public float getProbabilityAtTime(float time)
        {
                RuleTemplate tmpl = (RuleTemplate) (simulation.getStorage().getTemplate(templateId));
                float prob = tmpl.getProbabilityAtTime(time);
                return prob;
        }
		
        public void setSimulation(SimulationModel sim)
        {
                simulation = sim;
        }

        public void setRule(String act, String[] ic, String[] tc)
        {
                action = act;
                ifClause = ic;
                thenClause = tc;
        }

        // -- test the rule with a given action and particle
        public String test(Bond thisBond, Bond thatBond)
        {
                boolean passProb = true;
                boolean passPath = true;

                // -- test the probability

                float randval = rand.nextFloat();

                float arg = 0f;
                try
                {
                        arg = getProbabilityAtTime(sim.getElapsedTime());
                }
                catch (NumberFormatException e)
                {
                        return "fail";
                }
                if ( !compareFloats(randval, arg, "<") )
                {
                        passProb = false;
                }


                // -- test each of the conditions in turn (fail if any not met - ie. logical AND)
               for (int i=ifClause.length-1; i>=0; i--)
                {
                       //if (!(testCondition(ifClause[i], thisBond, thatBond))) { return false; }
                        String result = testCondition(ifClause[i], thisBond, thatBond);
                        if (result.equals("fail:error"))
                        {
                                return "fail";
                        }
                        else if (result.equals("fail:path"))
                        {
                                passPath = false;
                                passProb = false;
                        }
                }

                if (passProb)
                {
                        return "pass:all";
                }
                else
                {
                        if (passPath)
                        {
                                return "pass:path"; }
                        else
                        {
                                return "fail"; }
                }
        }

        String testCondition(String condition, Bond thisBond, Bond thatBond)
        {
                        String[] tokens = Functions.split(' ', condition);

                        if (tokens.length < 3) {return "fail:error";}

                        if ( comparePath(tokens, thisBond, thatBond))
                        {
                                return "pass:path";
                        }
                        else
                        {
                                return "fail:path";
                        }
        }

                boolean comparePath(String[] tokens, Bond thisBond, Bond thatBond)
                {
                        // split the path into its components
                        String[] pathTokens = Functions.split('.', tokens[0]);

                        String referenceType = "null";
                        Bond bond = null;
                        SimObject simObject = null;
                        Particle particle = null;

                        for (int i=0; i<pathTokens.length; i++)
                        {
                                if ( i == 0 && pathTokens[i].equals("this"))
                                {
                                        referenceType = "bond";
                                        bond = thisBond;
                                        continue;
                                }
                                if (i == 0 && pathTokens[i].equals("that"))
                                {
                                        referenceType = "bond";
                                        bond = thatBond;
                                        continue;
                                }
                                if (particle != null && pathTokens[i].equals("numBonded") && referenceType.equals("particle"))
                                {
                                        Vector bondIds = particle.getBondIds();
                                        int bonded = 0;
                                        for (int j=0; j<bondIds.size(); j++)
                                        {
                                                bond = (Bond) (simulation.getInstance((String)(bondIds.elementAt(j))));
                                                String bondedId = bond.getConnectedBond();
                                                if (!bondedId.equals(""))
                                                {
                                                        bonded++;
                                                }
                                        }
                                        int arg2;
                                        try
                                        {
                                                arg2 = Integer.parseInt(tokens[2]);
                                        }
                                        catch  (NumberFormatException e)
                                        {
                                                return false;
                                        }

                                        boolean returnval = compareInts(bonded, arg2, tokens[1]);
                                        return returnval;
                                }
                                if (simObject != null && pathTokens[i].equals("numBonded") && referenceType.equals("object"))
                                {
                                        Vector allParticleIds = simObject.getAllParticleIds();
                                        Vector bondIds = new Vector();
                                        for (int j=0; j<allParticleIds.size(); j++)
                                        {
                                                Particle partObj = (Particle) (simulation.getInstance((String)(allParticleIds.elementAt(j))));
                                                Vector particleBonds = partObj.getBondIds();
                                                for (int k=0; k<particleBonds.size(); k++)
                                                {
                                                        bondIds.addElement(particleBonds.elementAt(k));
                                                }
                                        }
                                        int bonded = 0;
                                        for (int j=0; j<bondIds.size(); j++)
                                        {
                                                bond = (Bond) (simulation.getInstance((String)(bondIds.elementAt(j))));
                                                String bondedId = bond.getConnectedBond();
                                                if (!bondedId.equals(""))
                                                {
                                                        bonded++;
                                                }
                                        }
                                        int arg2;
                                        try
                                        {
                                                arg2 = Integer.parseInt(tokens[2]);
                                        }
                                        catch  (NumberFormatException e)
                                        {
                                                return false;
                                        }

                                        boolean returnval = compareInts(bonded, arg2, tokens[1]);
                                        return returnval;
                                }
                                if (bond != null && pathTokens[i].equals("connected") && referenceType.equals("bond"))
                                {
                                        String bondId = bond.getConnectedBond();
                                        if (bondId.equals(""))
                                        {
                                                bond = null;
                                        }
                                        else
                                        {
                                                bond = (Bond) (simulation.getInstance(bondId));
                                        }
                                        continue;
                                }
                                if (bond != null && pathTokens[i].equals("particle") && referenceType.equals("bond"))
                                {
                                        referenceType = "particle";
                                        String particleId = bond.getParentParticle();
                                        if (particleId.equals(""))
                                        {
                                                particle = null;
                                        }
                                        else
                                        {
                                                particle = (Particle) (simulation.getInstance(particleId));
                                        }
                                        continue;
                                }
                                if (particle != null && pathTokens[i].equals("object") && referenceType.equals("particle"))
                                {
                                        referenceType = "object";
                                        String objectId = particle.getParentObjectId();
                                        if (objectId.equals(""))
                                        {
                                                simObject = null;
                                        }
                                        else
                                        {
                                                simObject = (SimObject) (simulation.getInstance(objectId));
                                        }
                                        continue;
                                }
                                if (simObject != null && pathTokens[i].length() > 9 && pathTokens[i].substring(0,9).equals("particle-")
                                    && referenceType.equals("object"))
                                {
                                        int index;
                                        try
                                        {
                                                index = Integer.parseInt(pathTokens[i].substring(9));
                                        }
                                        catch (NumberFormatException e)
                                        {
                                                continue;
                                        }

                                        referenceType = "particle";

                                        String[] particleIds = simObject.getParticleIds();
                                        if (particleIds.length <= index)
                                        {
                                                particle = null;
                                        }
                                        else
                                        {
                                                particle = (Particle)(simulation.getInstance(particleIds[index]));
                                        }
                                        continue;
                                }
                                if (particle != null && pathTokens[i].length() > 5 && pathTokens[i].substring(0,5).equals("bond-")
                                    && referenceType.equals("particle"))
                                {
                                        int index;
                                        try
                                        {
                                                index = Integer.parseInt(pathTokens[i].substring(5));
                                        }
                                        catch (NumberFormatException e)
                                        {
                                                continue;
                                        }

                                        referenceType = "bond";

                                        Vector bondIds = particle.getBondIds();
                                        if (bondIds.size() <= index)
                                        {
                                                bond = null;
                                        }
                                        else
                                        {
                                                bond = (Bond)(simulation.getInstance((String)(bondIds.elementAt(index))));
                                        }
                                        continue;
                                }
                                continue;
                        }

                        String componentStr;
                        if ( bond != null && referenceType.equals("bond"))
                        {
                                componentStr = bond.getTemplateId();
                        }
                        else if (particle != null && referenceType.equals("particle"))
                        {
                                componentStr = particle.getTemplateId();
                        }
                        else if (simObject != null && referenceType.equals("object"))
                        {
                                componentStr = simObject.getTemplateId();
                        }
                        else
                        {
                                componentStr = "";
                        }

                        return compareStrings(componentStr, tokens);
                }

                final boolean compareStrings(String component, String[] tokens)
                {
                        String comparator = tokens[1];
                        boolean success;
                        if (comparator.equals("!="))
                        {
                                success = true;
                        }
                        else
                        {
                                success = false;
                        }

                        for (int i=2; i<tokens.length; i++)
                        {
                                // -- the 'not null' operator
                                if (tokens[i].equals("*") && !component.equals(""))
                                {
                                        if (comparator.equals("=="))
                                        {
                                                success = true;
                                        }
                                        else if(comparator.equals("!="))
                                        {
                                                success = false;
                                        }
                                }
                                // -- the 'null' operator
                                else if (tokens[i].equals("null") && component.equals(""))
                                {
                                        if (comparator.equals("=="))
                                        {
                                                success = true;
                                        }
                                        else if(comparator.equals("!="))
                                        {
                                                success = false;
                                        }
                                }
                                // -- individual templateIds
                                else
                                {
                                        if (tokens[i].equals(component))
                                        {
                                                if (comparator.equals("=="))
                                                {
                                                        success = true;
                                                }
                                                else if (comparator.equals("!="))
                                                {
                                                        success = false;
                                                }
                                        }
                                }
                        }
                        return success;
                }

                boolean checkBondConnectedTemplate(String[] tokens, Bond thisBond, Bond thatBond)
                {
                        String particleId;
                        String templateId;
                        if (tokens[0].equals("this"))
                        {
                                String connectedBondId = thisBond.getConnectedBond();
                                if (connectedBondId.equals(""))
                                {
                                        templateId = "";
                                }
                                else
                                {
                                        Bond connectedBond = (Bond) (simulation.getInstance(connectedBondId));
                                        particleId = connectedBond.getParentParticle();
                                        templateId = simulation.getParticleTemplate(particleId);
                                }
                        }
                        else if (tokens[0].equals("that"))
                        {
                                String connectedBondId = thatBond.getConnectedBond();
                                if (connectedBondId.equals(""))
                                {
                                        templateId = "";
                                }
                                else
                                {
                                        Bond connectedBond = (Bond) (simulation.getInstance(connectedBondId));
                                        particleId = connectedBond.getParentParticle();
                                        templateId = simulation.getParticleTemplate(particleId);
                                }
                        }
                        else
                        {
                                return false;
                        }

                        boolean success;
                        if (tokens[2].equals("!="))
                        {
                                success = true;
                        }
                        else
                        {
                                success = false;
                        }

                        for (int i=3; i<tokens.length; i++)
                        {
                                // -- the 'not null' operator
                                if (tokens[i].equals("*"))
                                {
                                        if (tokens[2].equals("==") && !templateId.equals(""))
                                        {
                                                success = true;
                                        }
                                        else if(tokens[2].equals("!=") && !templateId.equals(""))
                                        {
                                                success = false;
                                        }
                                }
                                // -- the 'null' operator
                                else if (tokens[i].equals("null"))
                                {
                                        if (tokens[2].equals("==") && templateId.equals(""))
                                        {
                                                success = true;
                                        }
                                        else if(tokens[2].equals("!=") && templateId.equals(""))
                                        {
                                                success = false;
                                        }
                                }
                                // -- individual templateIds
                                else
                                {
                                        if (tokens[i].equals(templateId))
                                        {
                                                if (tokens[2].equals("=="))
                                                {
                                                        success = true;
                                                }
                                                else if (tokens[2].equals("!="))
                                                {
                                                        success = false;
                                                }
                                        }
                                }

                        }
                        return success;
                }

                boolean checkParticleTemplate(String[] tokens, Bond thisBond, Bond thatBond)
                {
                        String particleId;
                        String templateId;
                        if (tokens[0].equals("this"))
                        {
                                particleId = thisBond.getParentParticle();
                                templateId = simulation.getParticleTemplate(particleId);
                        }
                        else if (tokens[0].equals("that"))
                        {
                                particleId = thatBond.getParentParticle();
                                templateId = simulation.getParticleTemplate(particleId);
                        }
                        else
                        {
                                return false;
                        }

                        boolean success;
                        if (tokens[2].equals("!="))
                        {
                                success = true;
                        }
                        else
                        {
                                success = false;
                        }

                        for (int i=3; i<tokens.length; i++)
                        {
                                // -- the 'not null' operator
                                if (tokens[i].equals("*"))
                                {
                                        if (tokens[2].equals("==") && !templateId.equals(""))
                                        {
                                                success = true;
                                        }
                                        else if(tokens[2].equals("!=") && !templateId.equals(""))
                                        {
                                                success = false;
                                        }
                                }
                                // -- the 'null' operator
                                else if (tokens[i].equals("null"))
                                {
                                        if (tokens[2].equals("==") && templateId.equals(""))
                                        {
                                                success = true;
                                        }
                                        else if(tokens[2].equals("!=") && templateId.equals(""))
                                        {
                                                success = false;
                                        }
                                }
                                // -- individual templateIds
                                else
                                {
                                        if (tokens[i].equals(templateId))
                                        {
                                                if (tokens[2].equals("=="))
                                                {
                                                        success = true;
                                                }
                                                else if (tokens[2].equals("!="))
                                                {
                                                        success = false;
                                                }
                                        }
                                }

                        }
                        return success;
                }

        boolean compareInts(int arg1, int arg2, String comparator)
        {
                if (comparator.equals("=="))
                {
                        return (arg1 == arg2);
                }
                else if (comparator.equals("<"))
                {
                        return (arg1 < arg2);
                }
                else if (comparator.equals(">"))
                {
                        return (arg1 > arg2);
                }
                else if (comparator.equals("<="))
                {
                        return (arg1 <= arg2);
                }
                else if (comparator.equals(">="))
                {
                        return (arg1 >= arg2);
                }
                else if (comparator.equals("!="))
                {
                        return (arg1 != arg2);
                }
                return false;
        }

        boolean compareFloats(float arg1, float arg2, String comparator)
        {
                if (comparator.equals("=="))
                {
                        return (arg1 == arg2);
                }
                else if (comparator.equals("<"))
                {
                        return (arg1 < arg2);
                }
                else if (comparator.equals(">"))
                {
                        return (arg1 > arg2);
                }
                else if (comparator.equals("<="))
                {
                        return (arg1 <= arg2);
                }
                else if (comparator.equals(">="))
                {
                        return (arg1 >= arg2);
                }
                else if (comparator.equals("!="))
                {
                        return (arg1 != arg2);
                }
                return false;
        }
}