package code.model.structure;

import code.model.*;

import java.util.*;
import javax.vecmath.*;

public class SimObject
{
        private String id;
        private String templateId;
        private String[] particleIds;
        private String[] subObjectIds;

        private Vector3f centreOfMass;
        private AxisAngle4f orientation;
		private SimulationModel simulation;

        public SimObject(String i)
        {
                id = i;
                templateId = "";
                particleIds = new String[0];
                subObjectIds = new String[0];
				
                orientation = new AxisAngle4f();
                centreOfMass = new Vector3f();
        }
		
        public void setSimulation(SimulationModel sim)
        {
                simulation = sim;
        }

        // used to compute centres of mass
        public float getMass()
        {
                float mass = 0f;
                for (int i=0; i<particleIds.length; i++)
                {
                        String templateId = ((Particle)(simulation.getInstance(particleIds[i]))).getTemplateId();
                        mass += ((ParticleTemplate)(simulation.getTemplate(templateId))).getMass();
                }
                for (int i=0; i<subObjectIds.length; i++)
                {
                        mass += ((SimObject)(simulation.getInstance(subObjectIds[i]))).getMass();
                }
                return mass;
        }

        public void setId(String i)
        {
                id = i;
        }

        public String getId()
        {
                return id;
        }

        public void setSubObjectIds(String[] ids)
        {
                subObjectIds = ids;
        }

        public String[] getSubObjectIds()
        {
                return subObjectIds;
        }

        public void setParticleIds(String[] ids)
        {
                particleIds = ids;
        }

        public String[] getParticleIds()
        {
                return particleIds;
        }
		
        public Vector getAllParticleIds()
        {
                Vector allParts = new Vector();
                for (int i=0; i<particleIds.length; i++)
                {
                        allParts.addElement(particleIds[i]);
                }
                for (int i=0; i<subObjectIds.length; i++)
                {
                        SimObject obj = (SimObject) (simulation.getInstance(subObjectIds[i]));
                        Vector allSubObjParts = obj.getAllParticleIds();
                        for (int j=0; j<allSubObjParts.size(); j++)
                        {
                                allParts.addElement(allSubObjParts.elementAt(j));
                        }
                }
                return allParts;
        }

        public void setTemplateId(String i)
        {
                templateId = i;
        }

        public String getTemplateId()
        {
                return templateId;
        }

        public String toString()
        {
                String s = templateId + " (SimObjectInstance)";
                s += "\n\tparticles: ";
                for (int i=0; i<particleIds.length; i++)
                {
                        s += particleIds[i] + " ";
                }
                s += "\n\tsubObjects: ";
                for (int i=0; i<subObjectIds.length; i++)
                {
                        s += subObjectIds[i] + " ";
                }

                return s;
        }

        public Vector3f getCentreOfMass()
        {
                return centreOfMass;
        }

        public AxisAngle4f getOrientation()
        {
                return orientation;
        }

        public void setCentreOfMass(Vector3f com)
        {
                centreOfMass = com;
        }

        public void setOrientation(AxisAngle4f orient)
        {
                orientation.x = orient.x;
                orientation.y = orient.y;
                orientation.z = orient.z;
                orientation.angle = orient.angle;
        }

        public void translateCentreOfMass(Vector3f trans)
        {
                centreOfMass.x += trans.x;
                centreOfMass.y += trans.y;
                centreOfMass.z += trans.z;
        }
}