 /*
Copyright 2003 by the University of Edinburgh
Code by Dan Mossop.

Functions.java

Defines a series of functions which are required by the simulator code.

*/

package code.model.utils;

import javax.vecmath.*;

import java.util.*;

public class Functions
{
        public Functions()
        {}

        public static Vector3f rotatePoint(Vector3f point, Vector3f rotCentre, AxisAngle4f rotation)
        {
                // translate to the origin
                Vector3f transToOrigin = new Vector3f(point.x - rotCentre.x, point.y - rotCentre.y, point.z - rotCentre.z);

                // rotate the point about the origin
                Matrix4f rotMatrix = new Matrix4f();
                rotMatrix.set(rotation);
                Vector3f rotAtOrigin = new Vector3f();
                rotMatrix.transform(transToOrigin, rotAtOrigin);

                // translate back
                Vector3f finalPos = new Vector3f(rotAtOrigin.x + rotCentre.x, rotAtOrigin.y + rotCentre.y, rotAtOrigin.z + rotCentre.z);

                return finalPos;
        }

        public static Vector3f translatePoint(Vector3f point, Vector3f translation)
        {
                return (new Vector3f(point.x+translation.x, point.y+translation.y, point.z+translation.z));
        }

        public static Vector3f[] transformPoints(Vector3f[] points, Vector3f oldCOM,  Vector3f translation, AxisAngle4f rotation)
        {
                                Vector3f[] transformedPoints = new Vector3f[points.length];
                                for (int i=0; i<points.length; i++)
                                {
                                        /*
                                        // translate the particle to the origin.
                                        Vector3f transToOrigin = new Vector3f(points[i].x - oldCOM.x,
                                                                                                                                points[i].y - oldCOM.y, points[i].z - oldCOM.z);
                                        // rotate the particle about the object's centre of mass
                                        Matrix4f rotMatrix = new Matrix4f();
                                        rotMatrix.set(rotation);
                                        Vector3f rotAtOrigin = new Vector3f();
                                        rotMatrix.transform(transToOrigin, rotAtOrigin);
                                        // translate back
                                        transformedPoints[i] = new Vector3f(rotAtOrigin.x + oldCOM.x + translation.x,
                                                                                                        rotAtOrigin.y + oldCOM.y + translation.y,
                                                                                                        rotAtOrigin.z + oldCOM.z + translation.z);
                                         */
                                       Vector3f rotatedPoint = rotatePoint(points[i], oldCOM, rotation);
                                       transformedPoints[i] = translatePoint(rotatedPoint, translation);
                               }


                              return transformedPoints;
        }

        // -- calculate the cross product and dot product of two vectors
        // set axis to the cross product and return the angle between the vectors
        public static float getAxisAndAngle(Vector3f axis, Vector3f v1, Vector3f v2)
        {
                Vector3f a = new Vector3f();
                a.cross(v1, v2);
                a.normalize();

                if (Float.isNaN(a.x) || Float.isNaN(a.y) || Float.isNaN(a.z))
                {
                        axis.x = 0f;
                        axis.y = 1f;
                        axis.z = 0f;
                        return 0f;
                }
                else
                {
                        float angle = (float) Math.acos(v1.dot(v2));
                        axis.set(a);
                        return angle;
                }
        }

        public static boolean normalize(Vector3f v)
        {
                v.normalize();
                if (Float.isNaN(v.x) || Float.isNaN(v.y) || Float.isNaN(v.z))
                {
                        return false;
                }
                else
                {
                        return true;
                }
        }

        public static String[] split(char splitChar, String splitString)
        {
             Vector split = new Vector();
             int i = splitString.indexOf(splitChar+"");

             while (i != -1)
             {
                  split.addElement(splitString.substring(0, i));
                  splitString = splitString.substring(i+1);
                  i = splitString.indexOf(splitChar);
             }

             if (!(splitString.equals("")))
             {
                split.addElement(splitString);
             }

             Object[] splitArray = new Object[split.size()];
             split.copyInto(splitArray);

             String[] splitStringArray = new String[splitArray.length];
             for (int j=0; j<splitArray.length; j++)
             {
                splitStringArray[j] = (String) splitArray[j];
             }

             return splitStringArray;
        }

        public static AxisAngle4f combine(AxisAngle4f a1, AxisAngle4f a2)
        {
        		Vector3f[] points = new Vector3f[2];
        		points[0] = new Vector3f(0f, 1f, 0f);
        		points[1] = new Vector3f(1f, 0f, 0f);
        		
        		Vector3f[] newPoints = transformPoints(points, new Vector3f(), new Vector3f(), a1);
        		newPoints = transformPoints(newPoints, new Vector3f(), new Vector3f(), a2);
        		
        		Vector3f oldNormal, newNormal;
        		int index;
        		
        		if (points[0].equals(newPoints[0]) && points[1].equals(newPoints[1]))
        		{
        			return new AxisAngle4f(0f,1f,0f,0f);
        		}
        		else if (points[0].equals(newPoints[0]))
        		{
        			index = 1;
        		}
        		else 
        		{
        			index = 0;
        		}
        				
				oldNormal = new Vector3f(points[index].x,
										 points[index].y,
										 points[index].z);
				newNormal = new Vector3f(newPoints[index].x,
										 newPoints[index].y,
										 newPoints[index].z);
										 
				oldNormal.normalize();
				newNormal.normalize();
				
				Vector3f axis = new Vector3f();
				axis.cross(oldNormal, newNormal);
				
				float angle = oldNormal.angle(newNormal);
				
				if (angle == 0f)
				{
						axis = new Vector3f(0f, 1f, 0f);
				}
				return new AxisAngle4f(axis, angle);
    	}

        public static Object findRef(String refName, Vector lookup)
        {
                Vector names = (Vector)(lookup.elementAt(0));
                Vector references = (Vector)(lookup.elementAt(1));
                int index = names.indexOf(refName);

		if (index == -1)
		    { return null; }

                return references.elementAt(index);
        }
}