  /*
Copyright 2003 by the University of Edinburgh
Code by Dan Mossop.

BondControl.java

This file creates the bond control panel. This panel is used to view / edit the probabilities associated with the bonding rules.

*/
package gui;

import code.model.structure.*;
import code.CoordinateLayout.*;
import code.constants.*;

import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;
import java.io.*;

public class BondControl
{
        JFrame controlWindow;
        JPanel mainPanel;
        BondIdMenu bondIdMenu;
        Graph graphArea;
        ProbabilitySlider probSlide;

        public BondControl()
        {
                //--- Initialise the control window ---//

                // Get the current look and feel
                try
                {
                UIManager.setLookAndFeel(
                                UIManager.getSystemLookAndFeelClassName());
                } catch (Exception e) {}

                Font font = new Font("tahoma", Font.PLAIN, 10);

                // Create a JFrame
                controlWindow = new JFrame("Bond Control");
                controlWindow.setSize(640, 200);
                controlWindow.setLocation(new Point(0, 440));
                controlWindow.getContentPane().setLayout(new BorderLayout());
                controlWindow.addWindowListener(new WindowAdapter()
                {
                        public void windowClosing(WindowEvent e) {System.exit(0);}
                });
                controlWindow.setResizable(false);

                // Create the main panel
                mainPanel = new JPanel();
                mainPanel.setLayout(new BorderLayout());

                // -- Create the options panel
                JPanel optionsPanel = new JPanel();
                optionsPanel.setPreferredSize(new Dimension(200, 200));
                optionsPanel.setLayout(new CoordinateLayout(new Dimension(16,24),2,2));
                optionsPanel.setBorder(BorderFactory.createEtchedBorder());

                //TimeStepUnitMenu tuMenu = new TimeStepUnitMenu(timeUnitLabel);
                //tuMenu.setFont(font);
                JLabel bondIdTitle = new JLabel("Bond ID:");
                bondIdTitle.setFont(font);
                bondIdMenu = new BondIdMenu(this);
                bondIdMenu.setFont(font);

                JLabel controlMethodTitle = new JLabel("Control Method:");
                controlMethodTitle.setFont(font);
                JRadioButton manual = new JRadioButton("Manual");
                manual.setFont(font);
                manual.addActionListener(
                        new ActionListener(){
                                public void actionPerformed(ActionEvent e)
                                {
                                        probSlide.setManual(true);

                                        String[] rules = sim.getAllRuleTemplates();
                                        for (int i=0; i<rules.length; i++)
                                        {
                                                RuleTemplate rt = (RuleTemplate) (sim.getSimulationModel().getStorage().getTemplate(rules[i]));
                                                rt.setManual(true);
                                        }
                                        updateGraph();
                                }
                        }
                );
                JRadioButton auto = new JRadioButton("Auto");
                auto.setSelected(true);
                auto.setFont(font);
                auto.addActionListener(
                        new ActionListener(){
                                public void actionPerformed(ActionEvent e)
                                {
                                        probSlide.setManual(false);

                                        String[] rules = sim.getAllRuleTemplates();
                                        for (int i=0; i<rules.length; i++)
                                        {
                                                RuleTemplate rt = (RuleTemplate) (sim.getSimulationModel().getStorage().getTemplate(rules[i]));
                                                rt.setManual(false);
                                        }
                                        updateGraph();
                                }
                        }
                );
                ButtonGroup bGroup = new ButtonGroup();
                bGroup.add(manual);
                bGroup.add(auto);


                JLabel framesTitle = new JLabel("Graph Preview:");
                framesTitle.setFont(font);
                JLabel framesLab = new JLabel("Frames");
                framesLab.setFont(font);
                framesLab.setHorizontalAlignment(JLabel.LEFT);
                final JTextField framesVal = new JTextField("100");
                framesVal.setFont(font);
                framesVal.setHorizontalAlignment(JTextField.RIGHT);
                framesVal.addKeyListener(
                        new KeyListener()
                        {
                                public void keyPressed(KeyEvent e)
                                {
                                        try
                                        {
                                                int val = Integer.parseInt(framesVal.getText());
                                                graphArea.setNumFrames(val);
                                                updateGraph();
                                        }
                                        catch (Exception except)
                                        {
                                        }
                                }

                                public void keyTyped(KeyEvent e)
                                {}

                                public void keyReleased(KeyEvent e)
                                {
                                        try
                                        {
                                                int val = Integer.parseInt(framesVal.getText());
                                                graphArea.setNumFrames(val);
                                                updateGraph();
                                        }
                                        catch (Exception except)
                                        {
                                        }
                                }
                        }
                        );



                optionsPanel.add(bondIdTitle, new Coordinate(1,1,15,4));
                optionsPanel.add(bondIdMenu, new Coordinate(1,4,15,8));
                optionsPanel.add(controlMethodTitle, new Coordinate(1,8,15,12));
                optionsPanel.add(auto, new Coordinate(1,12,7,15));
                optionsPanel.add(manual, new Coordinate(7,12,15,15));
                optionsPanel.add(framesTitle, new Coordinate(1, 16, 15, 19));
                optionsPanel.add(framesVal, new Coordinate(1, 19, 5, 23));
                optionsPanel.add(framesLab, new Coordinate(6, 19, 15, 23));

                // -- Create the graph panel
                JPanel graphPanel = new JPanel();
                graphPanel.setPreferredSize(new Dimension(440, 200));
                graphPanel.setBorder(BorderFactory.createEtchedBorder());
                graphPanel.setLayout(new CoordinateLayout(new Dimension(14,16),2,2));

                JLabel prob = new JLabel("100%");
                prob.setFont(font);
                prob.setHorizontalAlignment(JLabel.RIGHT);
                probSlide = new ProbabilitySlider(font, this);
                graphArea = new Graph(this);

                // -- add components to the graph panel
                graphPanel.add(probSlide, new Coordinate(0, 0, 2, 15));
                graphPanel.add(prob, new Coordinate(0, 15, 1, 16));
                graphPanel.add(graphArea, new Coordinate(2, 0, 14, 15));


                // -- add components to the main panel
                mainPanel.add(optionsPanel, BorderLayout.WEST);
                mainPanel.add(graphPanel, BorderLayout.CENTER);

                controlWindow.getContentPane().add(mainPanel);
                controlWindow.setVisible(true);

                // -- render the graph
               // graphArea.drawBackground();
        }

        public void updateGraph()
        {
                updateGraph(bondIdMenu.getRuleId());
        }

        public void updateGraph(String ruleId)
        {
                if (graphArea != null)
                {
                        graphArea.update(ruleId);
                }
        }

        public void updateMenu()
        {
                bondIdMenu.update();
        }

        public void setSliderVal(float val)
        {
                probSlide.setSliderVal(val);
        }

        public String getRuleId()
        {
                return bondIdMenu.getRuleId();
        }
}

class ProbabilitySlider extends JSlider implements ChangeListener
{
        BondControl bondControl;
        boolean locked = false;

        public ProbabilitySlider(Font font, BondControl bc)
        {
                super();
                bondControl = bc;
                setEnabled(false);
                setOrientation(JSlider.VERTICAL);
                setPaintTicks(true);
                setPaintLabels(true);
                setMaximum(100);
                setMinimum(0);
                setMajorTickSpacing(10);
                setMinorTickSpacing(5);
                setValue(0);

                Dictionary dict = getLabelTable();
                for (Enumeration e = dict.elements() ; e.hasMoreElements() ;)
                {
                        ((JLabel) (e.nextElement())).setFont(font);
                }

                addChangeListener(this);

        }

       public void stateChanged(ChangeEvent e)
       {
                if (!locked)
                {
                        String ruleId = bondControl.getRuleId();
                        RuleTemplate rt = (RuleTemplate) (sim.getSimulationModel().getStorage().getTemplate(ruleId));
                        rt.setProbability( ((float) getValue()) / 100f );
                        bondControl.updateGraph();
                }
                else
                {
                        locked = false;
                }
       }

        public void setManual(boolean man)
        {
                setEnabled(man);
        }

        public boolean isManual()
        {
                return isEnabled();
        }

        public void setSliderVal(float val)
        {
                locked = true;
                setValue((int) (val * 100f));
        }
}

class BondIdMenu extends JComboBox implements ItemListener
{
        BondControl bondControl;

        public BondIdMenu(BondControl bc)
        {
                        super();
                        bondControl = bc;
                        setEditable(false);
                        addItemListener(this);
                        update();
        }

        public void update()
        {
                removeAllItems();
                String[] newItems = sim.getAllRuleTemplates();
                Arrays.sort(newItems);
                for (int i=0; i<newItems.length; i++)
                {
                        addItem(newItems[i]);
                }
                if (newItems.length > 0)
                {
                        setSelectedIndex(0);
                        bondControl.updateGraph(newItems[0]);
                }
                else
                {
                        setSelectedIndex(-1);
                        bondControl.updateGraph("");
                }
        }

        public void itemStateChanged(ItemEvent ie)
        {
                        String s = (String) ie.getItem();
                        bondControl.updateGraph(s);


                        RuleTemplate rt = (RuleTemplate) (sim.getSimulationModel().getStorage().getTemplate(s));
                        if (rt != null)
                        {
                                float elapsedTime = sim.getElapsedTime();
                                int val = (int) (100f * rt.getProbabilityAtTime(elapsedTime));
                                bondControl.setSliderVal(val);
                        }
        }

        public String getRuleId()
        {
                return (String) getSelectedItem();
        }
}

class Graph extends JPanel
{
        Vector points;
        int numFrames = 100;

        int leftOffset = 0;
        int rightOffset = 16;
        int topOffset = 7;
        int bottomOffset = 16;

        int numLinesX = 21;
        int numLinesY = 0;
        int majorIntervalX = 2;
        int majorIntervalY = 2;

        Graphics g;
        BondControl bondControl;

        Graph(BondControl bc)
        {
                bondControl = bc;
                points = new Vector();
        }

        public void setNumFrames(int num)
        {
                if (num > 1)
                {
                        numFrames = num;
                }
        }

        public void paint(Graphics graphics)
        {
                g = graphics;
                clear();
                drawGrid();
                drawLine(false);
        }

        void clear()
        {
                Dimension size = this.getSize();
                g.clearRect(0,0,(int) (size.getWidth()), (int) (size.getHeight()));
        }

        public void update(String ruleId)
        {
                int granularity = 1;
                int frms = numFrames;
                while (frms > 1000)
                {
                        frms /= 10;
                        granularity *= 10;
                }

                drawLine(true);
                points = new Vector();
                RuleTemplate rt = (RuleTemplate) (sim.getSimulationModel().getStorage().getTemplate(ruleId));

                float timeStep = sim.getTimeStep();
                float elapsedTime = sim.getElapsedTime();

                if (rt != null)
                {
                        for (int i=0; i<frms; i++)
                        {
                                float x = ((float)i- (float) (frms / 2)) / (float) (frms / 2);
                                float y = rt.getProbabilityAtTime(granularity * (i * timeStep) + elapsedTime);
                                if (i == 0)
                                {
                                        bondControl.setSliderVal(y);
                                }
                                y = y*2f -1f;
                                points.addElement(new PointF( x, y ));

                        }
                        drawLine(false);
                }
                Dimension size = this.getSize();
                paintImmediately(new Rectangle(0,0,(int) (size.getWidth()), (int) (size.getHeight())));
        }

        void drawLine(boolean erase)
        {
                Color lineCol;
                Color pointCol;

                if (erase)
                {
                        lineCol = new Color(64,64, 64);
                        pointCol = new Color(64,64, 64);
                }
                else
                {
                        lineCol = new Color(64,64, 255);
                        pointCol = new Color(64,255, 64);
                }

                Dimension size = this.getSize();
                int height = (int) size.getHeight()-bottomOffset;
                int width = (int) size.getWidth()-rightOffset;

                int xPixels = width - leftOffset;
                int yPixels = height; // - topOffset;
                //float oneTickX = (float) xPixels / 200f;
                //float oneTickY = (float) yPixels / 200f;

                for (int i=0; i<points.size()-1; i++)
                {
                        PointF start = (PointF) (points.elementAt(i));
                        PointF end = (PointF) (points.elementAt(i+1));

                        int x1 = leftOffset + (int)((float)xPixels * ((float)start.getX() + 1f)/2f);
                        int x2 = leftOffset + (int)((float)xPixels * ((float)end.getX() + 1f)/2f);
                        int y1 = topOffset + (int)((float)yPixels * (-(float)start.getY() + 1f)/2f);
                        int y2 = topOffset + (int)((float)yPixels * (-(float)end.getY() + 1f)/2f);

                        g.setColor(lineCol);
                        g.drawLine(x1, y1, x2, y2);

                        g.setColor(pointCol);
                        if (i == 0)
                        {
                                //g.fillRect(x2-1, y2-1, 3, 3);
                                g.fillRect(x1-1, y1-1, 3, 3);

                        }
                        //g.fillRect(x1-1, y1-1, 3, 3);
                }
        }

        void drawGrid()
        {
                Dimension size = this.getSize();
                int height = (int) size.getHeight()-bottomOffset;
                int width = (int) size.getWidth()-rightOffset;

                for (int i=0; i<=numLinesX-1; i++)
                {
                        if (i%majorIntervalX == 0)
                        {
                                g.setColor(new Color(165,165,165));
                        }
                        else
                        {
                                g.setColor(new Color(185,185,185));
                        }

                        if (i == 0 || i == numLinesX-1 || i == (int) ((numLinesX-1) / 2))
                        {
                                g.setColor(new Color(0, 0, 0));
                        }

                        g.drawLine(leftOffset, topOffset + i * height / (numLinesX-1), width, topOffset + i*height/(numLinesX-1));
                }

                for (int i=0; i<=numLinesY-1; i++)
                {
                        if (i%majorIntervalY == 0)
                        {
                                g.setColor(new Color(175,175,175));
                        }
                        else
                        {
                                g.setColor(new Color(150,150,150));
                        }
                        g.drawLine(leftOffset + i * (width - leftOffset) / (numLinesY-1), topOffset, leftOffset + i*(width-leftOffset)/(numLinesY-1), height+topOffset);
                }

                g.setColor(new Color(0, 0, 0));
                g.drawRect(0, 7, 350, 145);

        }
}

class PointF
{
        float x, y;

        PointF()
        {
                x=0f;
                y=0f;
        }

        PointF(float x, float y)
        {
                this.x = x;
                this.y = y;
        }

        float getX()
        {
                return x;
        }

        float getY()
        {
                return y;
        }

        void setX(float x)
        {
                this.x = x;
        }

        void setY(float y)
        {
                this.y = y;
        }
}
