 /*
Copyright 2003 by the University of Edinburgh
Code by Dan Mossop.

Control.java

This file creates the simulation control panel, which is used to load in the simulation files and control the speed / execution of the simulation

*/

package gui;

import code.model.structure.*;
import code.CoordinateLayout.*;
import code.constants.*;

import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;
import java.io.*;

public class Control
{
        JLabel timeUnitLabel;
        JLabel timeElapsedLabel;
        JLabel statusText;
        JLabel templateFileName;
        JLabel objectFileName;

        JButton startButton;
        JButton pauseButton;
        JButton restartButton;
        JButton clearButton;
        JButton templateButton;
        JButton objectButton;
        JTextField timeUnitVal;
        JTextField frameDelayVal;
        JFrame controlWindow;
        JPanel mainPanel;

        public Control()
        {
                //--- Initialise the control window ---//

                // Get the current look and feel
                try
                {
                UIManager.setLookAndFeel(
                                UIManager.getSystemLookAndFeelClassName());
                } catch (Exception e) {}

                Font font = new Font("tahoma", Font.PLAIN, 10);

                // Create a JFrame
                controlWindow = new JFrame("Main Control");
                controlWindow.setSize(200, 436);
                controlWindow.setLocation(new Point(0,0));
                controlWindow.getContentPane().setLayout(new BorderLayout());
                controlWindow.addWindowListener(new WindowAdapter()
                {
                        public void windowClosing(WindowEvent e) {System.exit(0);}
                });
                controlWindow.setResizable(false);

                // Create the control buttons
                startButton = new JButton("play");
                pauseButton = new JButton("pause");
                restartButton = new JButton("reset");
                clearButton = new JButton("clear");
                templateButton = new JButton("...");
                templateButton.setVerticalAlignment(JLabel.CENTER);
                objectButton = new JButton("...");
                objectButton.setVerticalAlignment(JLabel.CENTER);
                // start
                startButton.setEnabled(false);
                startButton.setMargin(new Insets(0,0,0,0));
                startButton.setFont(font);
                startButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                sim.setRunning(true);
                                startButton.setEnabled(false);
                                pauseButton.setEnabled(true);
                                restartButton.setEnabled(false);
                                clearButton.setEnabled(false);
                                templateButton.setEnabled(false);
                                objectButton.setEnabled(false);
                                setStatusText("Simulation running", false);
                        }
                });
                // pause
                pauseButton.setEnabled(false);
                pauseButton.setMargin(new Insets(0,0,0,0));
                pauseButton.setFont(font);
                pauseButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                sim.setRunning(false);
                                pauseButton.setEnabled(false);
                                startButton.setEnabled(true);
                                restartButton.setEnabled(true);
                                clearButton.setEnabled(true);
                                setStatusText("Simulation paused", false);
                        }
                });
                // restart
                restartButton.setEnabled(false);
                restartButton.setMargin(new Insets(0,0,0,0));
                restartButton.setFont(font);
                restartButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                restartButton.setEnabled(false);
                                clearButton.setEnabled(false);
                                startButton.setEnabled(false);
                                pauseButton.setEnabled(false);
                                sim.restart();
                                clearButton.setEnabled(true);
                                startButton.setEnabled(true);
                                templateButton.setEnabled(true);
                                objectButton.setEnabled(true);
                        }
                });
                // clear
                clearButton.setEnabled(false);
                clearButton.setMargin(new Insets(0,0,0,0));
                clearButton.setFont(font);
                clearButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                restartButton.setEnabled(false);
                                clearButton.setEnabled(false);
                                startButton.setEnabled(false);
                                pauseButton.setEnabled(false);
                                sim.clear();
                                templateButton.setEnabled(true);
                                objectButton.setEnabled(true);
                                startButton.setEnabled(false);
                        }
                });
                // template chooser
                final JFileChooser fcTemp = new JFileChooser(new File(Config.TEMPLATE_FILE_DIR));
                fcTemp.setMultiSelectionEnabled(false);
                templateButton.addActionListener(
                        new ActionListener()
                        {
                                public void actionPerformed(ActionEvent e)
                                {
                                        int templateResult = fcTemp.showOpenDialog(controlWindow);
                                        File templateFile = fcTemp.getSelectedFile();
                                        setStatusText("Loading...", false);
                                        repaintControl();
                                        if (templateResult == JFileChooser.APPROVE_OPTION)
                                        {
                                                sim.loadTemplateFile(templateFile.getPath());
                                                templateButton.setEnabled(false);
                                                clearButton.setEnabled(true);
                                        }
                                }
                        }
                        );
                // object chooser
                final JFileChooser fcObj = new JFileChooser(new File(Config.OBJECT_FILE_DIR));
                fcObj.setMultiSelectionEnabled(false);
                objectButton.addActionListener(
                        new ActionListener()
                        {
                                public void actionPerformed(ActionEvent e)
                                {
                                        int objectResult = fcObj.showOpenDialog(controlWindow);
                                        File objectFile = fcObj.getSelectedFile();
                                        setStatusText("Loading...", false);
                                        repaintControl();
                                        if (objectResult == JFileChooser.APPROVE_OPTION)
                                        {
                                                sim.loadObjectFile(objectFile.getPath());
                                                objectButton.setEnabled(false);
                                                clearButton.setEnabled(true);
                                        }
                                }
                        }
                        );

                // Create the filename panel
                JPanel filePanel = new JPanel();
                filePanel.setLayout(new CoordinateLayout(new Dimension(16,16),2,2));
                filePanel.setBorder(BorderFactory.createEtchedBorder());
                JLabel templateTitle = new JLabel("Template File:");
                templateTitle.setFont(font);
                templateFileName = new JLabel("None");
                templateFileName.setFont(font);
                JLabel objectTitle = new JLabel("Object File:");
                objectTitle.setFont(font);
                objectFileName = new JLabel("None");
                objectFileName.setFont(font);

                filePanel.add(templateTitle, new Coordinate(1,1,10,4));
                filePanel.add(templateFileName, new Coordinate(1,4,10,7));
                filePanel.add(templateButton, new Coordinate(12,2,15,7));
                filePanel.add(objectTitle, new Coordinate(1,8,10,11));
                filePanel.add(objectFileName, new Coordinate(1,11,10,14));
                filePanel.add(objectButton, new Coordinate(12,9,15,14));

                // Create the timestep panel
                JPanel timePanel = new JPanel();
                timePanel.setLayout(new CoordinateLayout(new Dimension(16,24),2,2));
                timePanel.setBorder(BorderFactory.createEtchedBorder());
                timeUnitLabel = new JLabel("");
                timeUnitLabel.setFont(font);
                timeElapsedLabel = new JLabel("0.0", JLabel.RIGHT);
                timeElapsedLabel.setFont(font);
                TimeStepUnitMenu tuMenu = new TimeStepUnitMenu(timeUnitLabel);
                tuMenu.setFont(font);
                JLabel timeStepTitle = new JLabel("Timestep per frame:");
                timeStepTitle.setFont(font);
                JLabel elapsedTitle = new JLabel("Elapsed Time:");
                elapsedTitle.setFont(font);
                timeUnitVal = new JTextField("1");
                timeUnitVal.setFont(font);
                timeUnitVal.setHorizontalAlignment(JTextField.RIGHT);
                timeUnitVal.addKeyListener(
                        new KeyListener()
                        {
                                public void keyPressed(KeyEvent e)
                                {
                                        try
                                        {
                                                float val = Float.parseFloat(timeUnitVal.getText());
                                                sim.setTimeValue(val);
                                        }
                                        catch (Exception except)
                                        {
                                        }
                                }

                                public void keyTyped(KeyEvent e)
                                {}

                                public void keyReleased(KeyEvent e)
                                {
                                        try
                                        {
                                                float val = Float.parseFloat(timeUnitVal.getText());
                                                sim.setTimeValue(val);
                                        }
                                        catch (Exception except)
                                        {
                                        }
                                }
                        }
                        );

                JLabel frameDelayTitle = new JLabel("Frame Delay:");
                frameDelayTitle.setFont(font);
                JLabel frameDelayUnits = new JLabel("Milliseconds");
                frameDelayUnits.setFont(font);
                frameDelayUnits.setHorizontalAlignment(JLabel.LEFT);
                frameDelayVal = new JTextField("0");
                frameDelayVal.setFont(font);
                frameDelayVal.setHorizontalAlignment(JTextField.RIGHT);
                frameDelayVal.addKeyListener(
                        new KeyListener()
                        {
                                public void keyPressed(KeyEvent e)
                                {
                                        try
                                        {
                                                int val = Integer.parseInt(frameDelayVal.getText());
                                               sim.getSimulationModel().setFrameDelay(val);
                                        }
                                        catch (Exception except)
                                        {
                                        }
                                }

                                public void keyTyped(KeyEvent e)
                                {}

                                public void keyReleased(KeyEvent e)
                                {
                                        try
                                        {
                                                int val = Integer.parseInt(frameDelayVal.getText());
                                               sim.getSimulationModel().setFrameDelay(val);
                                        }
                                        catch (Exception except)
                                        {
                                        }
                                }
                        }
                        );



                timePanel.add(timeStepTitle, new Coordinate(1,1,15,4));
                timePanel.add(timeUnitVal, new Coordinate(1,4,5,8));
                timePanel.add(tuMenu, new Coordinate(5,4,15,8));
                timePanel.add(elapsedTitle, new Coordinate(1,8,15,12));
                timePanel.add(timeUnitLabel, new Coordinate(7,12,15,15));
                timePanel.add(timeElapsedLabel, new Coordinate(1,12,7,15));
                timePanel.add(frameDelayTitle, new Coordinate(1, 16, 15, 19));
                timePanel.add(frameDelayVal, new Coordinate(1, 19, 5, 23));
                timePanel.add(frameDelayUnits, new Coordinate(6, 19, 15, 23));

                // create the status panel
                JPanel statusPanel = new JPanel();
                statusPanel.setLayout(new CoordinateLayout(new Dimension(8,8),2,2));
                TitledBorder statusBorder = BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "status");
                statusBorder.setTitleFont(font);
                statusPanel.setBorder(statusBorder);
                statusBorder.setTitleJustification(TitledBorder.LEFT);
                statusText = new JLabel("No simulation loaded", JLabel.CENTER);
                statusText.setFont(font);
                statusPanel.add(statusText, new Coordinate(1,1,7,7));

                // Create a JPanel for the objects
                mainPanel = new JPanel();

                mainPanel.setLayout(new CoordinateLayout(new Dimension(23, 29), 2, 2));

                mainPanel.add(startButton, new Coordinate(1,21,6,23));
                mainPanel.add(pauseButton, new Coordinate(6,21,11,23));
                mainPanel.add(restartButton, new Coordinate(11,21,16,23));
                mainPanel.add(clearButton, new Coordinate(17,21,22,23));

                mainPanel.add(filePanel, new Coordinate(1,1,22,8));
                mainPanel.add(timePanel, new Coordinate(1,9,22,20));
                mainPanel.add(statusPanel, new Coordinate(1,24,22,28));

                controlWindow.getContentPane().add(mainPanel);

                controlWindow.setVisible(true);
        }

        public void repaintControl()
        {
                mainPanel.paintImmediately(mainPanel.getBounds());
        }

        public void setTemplateFileName(String filename)
        {
                templateFileName.setText(filename);
                if (!(templateFileName.equals("None") || objectFileName.equals("None")))
                {
                        startButton.setEnabled(true);
                }
        }

        public void setObjectFileName(String filename)
        {
                objectFileName.setText(filename);
                if (!(templateFileName.equals("None") || objectFileName.equals("None")))
                {
                        startButton.setEnabled(true);
                }
        }

        public void setStatusText(String text, boolean isError)
        {
                if (isError)
                {
                        statusText.setForeground(new Color(1f,0f,0f));
                        startButton.setEnabled(false);
                }
                else
                {
                        statusText.setForeground(new Color(0f,0f,0f));
                }
                statusText.setText(text);
        }

        public void setElapsedTime(float et)
        {
                timeElapsedLabel.setText(""+(int)(et / sim.getTimeUnit()));
        }
}


class TimeStepUnitMenu extends JComboBox implements ItemListener
{
	JLabel simElapseUnit;

	public TimeStepUnitMenu(JLabel seu)
	{
		super();
		simElapseUnit = seu;

		addItem("Seconds");
		addItem("Milliseconds");
		addItem("Microseconds");
		addItem("Nanoseconds");
		addItem("Picoseconds");
		addItem("Femtoseconds");

		setEditable(false);

		setSelectedIndex(2);
                simElapseUnit.setText("Microseconds");
		sim.setTimeUnit((float) Math.pow(10.0, -6.0));

		addItemListener(this);
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();

		if (s.equals("Seconds")) { sim.setTimeUnit(1f); }
		if (s.equals("Milliseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Microseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -6.0)); }
		if (s.equals("Nanoseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -9.0)); }
		if (s.equals("Picoseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -12.0)); }
		if (s.equals("Femtoseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -15.0)); }

		simElapseUnit.setText(s);
                sim.setElapsedTime();
	}

	public void refresh()
	{
		String s = (String) getSelectedItem();

		if (s.equals("Seconds")) { sim.setTimeUnit(1f); }
		if (s.equals("Milliseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Microseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -6.0)); }
		if (s.equals("Nanoseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -9.0)); }
		if (s.equals("Picoseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -12.0)); }
		if (s.equals("Femtoseconds"))
			{ sim.setTimeUnit((float) Math.pow(10.0, -15.0)); }
	}

}

class TimeStepValueTextField extends JTextField implements KeyListener
{

	public TimeStepValueTextField()
	{
		super(6);
		setText("1");
		sim.setTimeValue(1f);
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}

	public void keyPressed(KeyEvent ke)
	{}

	public void keyTyped(KeyEvent ke)
	{}

	public void keyReleased(KeyEvent ke)
	{
		try
		{
			float f = (float) (Float.parseFloat(getText()));
			sim.setTimeValue(f);
		}
		catch (NumberFormatException e)
		{}
	}
}