/*
Copyright 2003 by the University of Edinburgh
Created by Dan Mossop.

sim.java

This is the file containing main() method which is run from the command line.
It is responsible for the creation of the simulation and interface and maintaining these during runtime.
*/

package gui;

import code.model.structure.*;  // Particle
import code.model.display.Display;    // Display
import code.constants.*;
import code.update.Evolver;
import code.model.SimulationModel; // SimulationModel

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;
import java.io.*;

import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;


public class sim extends JPanel
{
        static float simulationSize = 2000f * Units.ANGSTROMS_PER_METRE;
        static float displayScale = 10f/simulationSize;
        static Display display;
        static boolean isRunning = false;
        static float timeUnit = (float) Math.pow(10.0, -6.0);
        static float timeVal = 1f;
        static float timeStep = timeUnit * timeVal;
        static float elapsedTime = 0f;

        static Control control;
        static BondControl bondControl;
        static SimulationModel simulationModel;
        static Evolver evolver;

        static String templateFile = "";
        static String objectFile = "";

        public sim()
        {}

        public static void main(String[] args) throws Exception
        {
                        simulationModel = new SimulationModel(new Point(200,0), new Dimension(440,440));
                        simulationModel.setSimulationSize(simulationSize);

                        // create the updating object
                        evolver = new Evolver(simulationModel);
                        evolver.setTimestep(timeStep);

                        simulationModel.setEvolver(evolver);

                        // Create the control window
                        control = new Control();
                        bondControl = new BondControl();

                        while (true)
                        {
                                if (isRunning)
                                {
                                simulationModel.setRunning(true);
                                Thread.currentThread().sleep(100);
                                }
                                else
                                {
                                        simulationModel.setRunning(false);
                                        Thread.currentThread().sleep(100);
                                }

                        }
        }

        public static String[] getAllRuleTemplates()
        {
                return simulationModel.getAllRuleTemplates();
        }

        public static void setSimulationSize(float size)
        {
                simulationModel.setSimulationSize(size);
        }

        public static void updateElapsedTime()
        {
                elapsedTime += timeStep;
                control.setElapsedTime(elapsedTime);
                updateGraph();
        }

        public static void updateGraph()
        {
                if (bondControl != null)
                {
                        bondControl.updateGraph();
                }
        }

        public static SimulationModel getSimulationModel()
        {
                return simulationModel;
        }

        public static void setTimeUnit(float tu)
        {
                timeUnit = tu;
                timeStep = timeUnit * timeVal;
                evolver.setTimestep(timeStep);
                updateGraph();
        }

        public static void setTimeValue(float tv)
        {
                timeVal = tv;
                timeStep = timeUnit * timeVal;
                evolver.setTimestep(timeStep);
                updateGraph();
        }

        public static float getTimeUnit()
        {
                return timeUnit;
        }

        public static float getTimeStep()
        {
                return timeStep;
        }

        public static float getElapsedTime()
        {
                return elapsedTime;
        }

        public static void setElapsedTime()
        {
                control.setElapsedTime(elapsedTime);
        }

        public static void setRunning(boolean run)
        {
                isRunning = run;
        }

        public static boolean getRunning()
        {
                return isRunning;
        }

        public static void tryToLoadSimulation()
        {
                if (templateFile.equals("") && objectFile.equals(""))
                {
                        control.setStatusText("No simulation loaded", false);
                        return;
                }
                else if (templateFile.equals(""))
                {
                        control.setStatusText("Select template file", false);
                        return;
                }
                else if (objectFile.equals(""))
                {
                        control.setStatusText("Select object file", false);
                        return;
                }
                else
                {
                       try
                       {
                                control.setStatusText("Loading...", false);
                                simulationModel.loadFile(templateFile);
                                simulationModel.loadFile(objectFile);
                                simulationModel.display();
                                control.setStatusText("Simulation ready", false);
                       }
                       catch (Exception e)
                       {
                                control.setStatusText("Error in input files", true);
                                return;
                       }
                       bondControl.updateMenu();
                }


        }

        public static void restart()
        {
                elapsedTime = 0f;
                control.setElapsedTime(0f);
                newModel();
                tryToLoadSimulation();
        }

        public static void clear()
        {
                elapsedTime = 0f;
                control.setElapsedTime(0f);
                templateFile="";
                objectFile="";
                control.setTemplateFileName("None");
                control.setObjectFileName("None");
                newModel();
        }

        public static void loadTemplateFile(String filename)
        {
                templateFile = filename;
                control.setTemplateFileName((new File(filename)).getName());
                tryToLoadSimulation();
        }

        public static void loadObjectFile(String filename)
        {
                objectFile = filename;
                control.setObjectFileName((new File(filename)).getName());
                tryToLoadSimulation();
        }

        public static void newModel()
        {
                isRunning = false;
                try
                {
                        Thread.currentThread().sleep(50);
                }
                catch (InterruptedException e)
                {
                }
                Point loc = simulationModel.getLocation();
                Dimension dim = simulationModel.getDimensions();
                int fdelay = simulationModel.getFrameDelay();
                simulationModel.clear();
                simulationModel.dispose();
                simulationModel = new SimulationModel(loc, dim);
                simulationModel.setSimulationSize(simulationSize);
                simulationModel.setFrameDelay(fdelay);
                evolver = new Evolver(simulationModel);
                evolver.setTimestep(timeStep);
                simulationModel.setEvolver(evolver);
                bondControl.updateMenu();
        }
}




